import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.*;

public class NimWorld extends World
{
    private Match[] heap1 = { new Match(), new Match(), new Match() };
    private Match[] heap2 = { new Match(), new Match(), new Match(), new Match() };
    private Match[] heap3 = { new Match(), new Match(), new Match(), new Match(), new Match() };
    
    private boolean player, gameOver;

    public NimWorld() {    
        super(500, 500, 1); 
        GreenfootImage bg = new GreenfootImage(getWidth(), getHeight());
        bg.setColor(Color.green);
        bg.fill();
        setBackground(bg);
        prepare();
    }
    
    private void prepare() {
        GreenfootImage bg = getBackground();
        Font f = new Font("Arial", Font.BOLD, 70);
        bg.setFont(f);
        bg.setColor(Color.black);
        bg.drawString("Who starts?", getWidth()/2-190, 150);
        addObject(new Button("Player", 0), getWidth()/2-100, getHeight()/2);
        addObject(new Button("Computer", 1), getWidth()/2+100, getHeight()/2);
    }
    
    public void startGame(boolean p) {
        removeObjects(getObjects(Button.class));
        player = p;
        gameOver = false;
        GreenfootImage bg = getBackground();
        bg.setColor(Color.green);
        bg.fill();
        setBackground(bg);
        for(int i=0; i < heap1.length; i++) {
            addObject(heap1[i], (getWidth()/2-100)+i*100, 100);
        }
        for(int i=0; i < heap2.length; i++) {
            addObject(heap2[i], (getWidth()/2-150)+i*100, 250);
        }
        for(int i=0; i < heap3.length; i++) {
            addObject(heap3[i], (getWidth()/2-200)+i*100, 400);
        }
        addObject(new Player(), 0, 0);
        addObject(new AI(), 0, 0);
        addObject(new Button("End turn", 2), getWidth()/2, 15);
    }
    
    public void endTurn() {
        checkGameOver();
        if(!gameOver) {
            boolean changed = false;
            if(player && !changed) {
                addObject(new Alert("Computer"), getWidth()/2, getHeight()/2);
                Greenfoot.delay(50);
                removeObjects(getObjects(Alert.class));
                player = false;
                changed = true;
            }
            if(!player && !changed) {
                addObject(new Alert("Player"), getWidth()/2, getHeight()/2);
                Greenfoot.delay(50);
                removeObjects(getObjects(Alert.class));
                player = true;
                changed = true;
            }
        }
    }
    
    private void checkGameOver() {
        boolean allHidden = true;
        for(int i=0; i < heap1.length; i++) {
            if(!heap1[i].isHidden()) {
                allHidden = false;
            }
        }
        for(int i=0; i < heap2.length; i++) {
            if(!heap2[i].isHidden()) {
                allHidden = false;
            }
        }
        for(int i=0; i < heap3.length; i++) {
            if(!heap3[i].isHidden()) {
                allHidden = false;
            }
        }
        if(allHidden) {
            gameOver = true;
            if(player) {
                addObject(new Alert("Player wins!"), getWidth()/2, getHeight()/2);
            }
            else {
                addObject(new Alert("Computer wins!"), getWidth()/2, getHeight()/2);
            }
            Greenfoot.stop();
        }
    }
    
    public boolean turn() {
        return player;
    }
    
    public String[] getRows() {
        String[] sum = new String[3];
        int s = 0;
        for(int i=0; i < heap1.length; i++) {
            if(!heap1[i].isHidden()) {
                s++;
            }
        }
        sum[0] = Integer.toBinaryString(s);
        for(int i=0; i < 3; i++) {
            if(sum[0].length() < 3) {
                sum[0] = "0" + sum[0];
            }
        }
        s = 0;
        for(int i=0; i < heap2.length; i++) {
            if(!heap2[i].isHidden()) {
                s++;
            }
        }
        sum[1] = Integer.toBinaryString(s);
        for(int i=0; i < 3; i++) {
            if(sum[1].length() < 3) {
                sum[1] = "0" + sum[1];
            }
        }
        s = 0;
        for(int i=0; i < heap3.length; i++) {
            if(!heap3[i].isHidden()) {
                s++;
            }
        }
        sum[2] = Integer.toBinaryString(s);
        for(int i=0; i < 3; i++) {
            if(sum[2].length() < 3) {
                sum[2] = "0" + sum[2];
            }
        }
        return sum;
    }
    
    public Match[] getHeap(int i) {
        if(i == 0) {
            return heap1;
        }
        else if(i == 1) {
            return heap2;
        }
        else if(i == 2) {
            return heap3;
        }
        else {
            return null;
        }
    }
}
